# include <stdio.h>
# include <stdlib.h>
# include <math.h>
const double eps = 1e-6;

double f(double x)
{
    return x*(x-3)*(x-3)+2;
}

/** Inequality for double values **/
bool notEqual(double a,double b)
{
   return fabs(a-b)>eps;
}
/*
 * stopflag=0, did not find root
 * stopflag=1, root was found
*/
void Brent(double &a,double &b,int &stopflag)
{
    double fa = f(a);
    double fb = f(b);
    double fc;
    double  c,d=0;
    double s;//step;
    bool mflag =false;
    int iter = 0;
    if(fa * fb>0)
    {
        stopflag=0;
        return;
    }
    if(fabs(fa)>fabs(fb))
    {
        /**
         *  swap**/
        double t = a;
        a = b;
        b= t;
    }
    c=a;
    fc = fa;
    mflag =true;
    do
    {

        if(notEqual(fa,fb) && notEqual(fb,fc))
        {
            s=(a*fb*fc)/((fa-fb)*(fa-fc))+
                    (b*fa*fc)/((fb-fa)*(fb-fc))+
                    (c*fa*fb)/((fc-fa)*(fc-fb));
        }
        else
        {
            s=b-fb*(b-a)/(fb-fa);
        }
        if(!(s>=(3*a+b)/4 && s<=b)
          || (mflag && fabs(s-b)>=fabs(b-c)/2.0)
          || (!mflag && fabs(s-b)>=fabs(c-d)/2.0)
          || (mflag && fabs(b-c)<eps)
          || (!mflag && fabs(c-d)<eps))
        {
            s=(a+b)/2.0;
        }
        else
            mflag =false;
        double fs= f(s);
        d=c;
        c=b;
        fc = f(c);
        if(fa*fs<0)
        {
            b = s;
            fb=fs;
        }
        else
        {
            a=s;
            fa=fs;
        }
        if(fabs(fa)<fabs(fb))
        {
            double t= a;
            a= b;
            b=t;
            t=fa;
            fa=fb;
            fb=t;
        }
        iter++;
        printf("Iter = %4d A = %20.10lg B = %20.10lg\n",iter,a,b);
        if(fabs(fb)<eps || fabs(fs)<eps || fabs(b-a)<eps) {
            stopflag=1;
            break;
        }
    }while(true);
}
int main()
{
    double a= -4;
    double b = 4;
    int stopflag =0;
    Brent(a,b,stopflag);
    return 0;
}
